
/*
 * $Id: ServerRep.cc,v 1.2.4.6 2000/01/12 00:05:35 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "polygraph.h"

#include "ServerRep.h"
#include "ErrorMgr.h"
#include "OLog.h"
#include "ILog.h"

#include "gadgets.h"
#include "polyErrors.h"


int ServerRep::TheNewOidsPrefetchDepth = -1;
int ServerRep::TheNewOidsPerReqMax = -1;

void ServerRep::ConfigureNids(int prefetchLmt, int perReqMax) {
	TheNewOidsPrefetchDepth = prefetchLmt;
	TheNewOidsPerReqMax = perReqMax;
}

ServerRep::ServerRep(const NetAddr &anAddr): theAddr(anAddr),
	theLastNewOid(0), theSecOid(0), 
	theAccCount(0), theWorkSetCap(-1), thePendCnt(0) {
	if (TheNewOidsPrefetchDepth > -1) {
		theNewOids.resize(TheNewOidsPrefetchDepth);
		theFailedOids.resize(TheNewOidsPrefetchDepth);
	}
}

int ServerRep::oidShortage() {
	//cerr << here << thePendCnt << " + " << theNewOids.count() << " = " << shortage << endl;
	const int shortage = theNewOids.capacity() - (thePendCnt + theNewOids.count());
	const int req_size = Min(TheNewOidsPerReqMax, shortage);
	thePendCnt += req_size;
	return req_size;
}

int ServerRep::newOid() {
	Assert(haveNewOids());
	return theLastNewOid = theNewOids.dequeue();
}

int ServerRep::failedOid() {
	Assert(haveFailedOids());
	return theFailedOids.dequeue();
}

void ServerRep::noteNewOids(int oid, int askCnt, int gotCnt) {
	Assert(!gotCnt || oid > 0);

	thePendCnt -= askCnt;
	if (thePendCnt < 0)
		thePendCnt = 0;

	while (gotCnt-- > 0) {
		if (theNewOids.full()) {
			ReportError(errNidRepFlood);
			break;
		}
		theNewOids.enqueue(oid++);
	}
}

void ServerRep::noteFailedOid(int oid) {
	Assert(oid > 0);
	if (oid <= theSecOid) // no need to report
		return;

	if (theFailedOids.full())
		ReportError(errFidFlood);
	else
		theFailedOids.enqueue(oid);
}

void ServerRep::syncSrvInfo(const UniqId &aSrvId, int aSecOid) {
	if (!theId)
		theId = aSrvId;
	else
	if (theId != aSrvId) {
		ReportError(errSrvChangedWid);
		theNewOids.reset();
		theId = aSrvId;
		theSecOid = 0;
		thePendCnt = 0;
	}

	if (aSecOid > theSecOid)
		theSecOid = aSecOid;
}

void ServerRep::noteNoNewOids() {
	if (thePendCnt + theNewOids.count() >= theNewOids.capacity())
		theNewOids.resize(Max(1, 2*theNewOids.capacity()));
}

void ServerRep::freezeWorkSetCap() {
	// do not allow temporary lockup with theSecOid to affect WS
	theWorkSetCap = Max(theSecOid, theLastNewOid);
}

void ServerRep::store(OLog &log) const {
	log
		<< theAddr
		<< theId
		<< theSecOid
		<< theWorkSetCap
		;
}

void ServerRep::load(ILog &log) {
	log
		>> theAddr
		>> theId
		>> theSecOid
		>> theWorkSetCap
		;
}
