
/*
 * $Id: httpHdrs.h,v 1.7.2.2 2000/01/11 23:59:49 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef HTTP_HEADERS_H
#define HTTP_HEADERS_H

#include <iostream.h>

#include "Array.h"
#include "UniqId.h"
#include "Oid.h"
#include "LibInit.h"

class MsgHdrParsTab;

class MsgHdr {
	public:
		typedef bool (MsgHdr::*Parser)(const char *buf, const char *eoh);

	public:
		MsgHdr(const MsgHdrParsTab &aTab);
		virtual ~MsgHdr() {}

		virtual void reset();

		bool keepAlive() const;

		bool parse(const char *buf, Size sz);

	protected:
		static void Configure(MsgHdrParsTab &tab);
		static int AddParser(const String &field, Parser parser, MsgHdrParsTab &tab);

	protected:
		void parseFields();
		bool parseUniqId(UniqId &id, const char *buf, const char *eoh);

		virtual bool parseRLine(const char *buf, const char *eol) = 0;

		// can be used by both requests and replies
		bool parseContLen(const char *buf, const char *eoh);
		bool parseXXact(const char *buf, const char *eoh);
		bool parseConnection(const char *buf, const char *eoh);

		// these have to be here so we can register them in req
		virtual bool parseIMS(const char *buf, const char *eoh);
		virtual bool parseXOidShortage(const char *buf, const char *eoh);
		virtual bool parseXFailedOid(const char *buf, const char *eoh);

		// these have to be here so we can register them in rep
		virtual bool parsePragma(const char *buf, const char *eoh);
		virtual bool parseCControl(const char *buf, const char *eoh);
		virtual bool parseXSrvWid(const char *buf, const char *eoh);
		virtual bool parseXSrvOids(const char *buf, const char *eoh);

	public:
		const MsgHdrParsTab &theParsTab;

		Size theHdrSize;
		Size theContSize;
		UniqId theXactId;
		double theHttpVer;
		enum { kaNo, kaDefault, kaYes } theKeepAliveConn;

	protected:
		Array<const char *> theFields; // positions of header fields
		const char *theBufBeg, *theBufEnd;
		const char *theSrchPtr;        // parse buffer iterator
		enum { ssFirst, ssSkip, ssFound } theSrchState;
};

class ReqHdr: public MsgHdr {
	public:
		static void Configure();
		static void Clean();
		
	public:
		ReqHdr();

		virtual void reset();

		bool foreign() const { return theOidRec.oid <= 0 || !theXactId; }

	protected:
		virtual bool parseRLine(const char *buf, const char *eol);
		virtual bool parseIMS(const char *buf, const char *eoh);
		virtual bool parseXOidShortage(const char *buf, const char *eoh);
		virtual bool parseXFailedOid(const char *buf, const char *eoh);

	protected:
		static MsgHdrParsTab *TheParsTab;

	public:
		OidRec theOidRec;
		Time theIMS;       // If-Modified-Since
		int theOidShortage;
		int theFailedOid;
};

class RepHdr: public MsgHdr {
	public:
		static void Configure();
		static void Clean();
		
	public:
		RepHdr();

		virtual void reset();

		bool domestic() const { return theXactId && theSrvWid; }
		bool foreign() const { return !domestic(); }

	protected:
		virtual bool parseRLine(const char *buf, const char *eol);

		virtual bool parsePragma(const char *buf, const char *eoh);
		virtual bool parseCControl(const char *buf, const char *eoh);
		virtual bool parseXSrvWid(const char *buf, const char *eoh);
		virtual bool parseXSrvOids(const char *buf, const char *eoh);

	protected:
		static MsgHdrParsTab *TheParsTab;

	public:
		UniqId theSrvWid; // server world id
		int theSecOid;    // "secured" oid
		int theNewOid;    // as advertised by the server
		int theNewOidCnt;
		bool isCachable;
};

inline ostream &crlf(ostream &os) { return os.write("\r\n", 2); }

LIB_INITIALIZER(HttpHdrsInit)

#endif
