
/*
 * $Id: HostsBasedSym.cc,v 1.1.2.1 1999/10/31 03:09:59 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "pgl.h"

#include "String.h"
#include "PglRec.h"
#include "PglArraySym.h"
#include "PglNetAddrSym.h"
#include "HostsBasedSym.h"


HostsBasedSym::HostsBasedSym(const String &aType): RecSym(aType, new PglRec) {
	theRec->bAdd("addr[]", "hosts", 0);
}

HostsBasedSym::HostsBasedSym(const String &aType, PglRec *aRec): RecSym(aType, aRec) {
}

bool HostsBasedSym::isA(const String &type) const {
	return RecSym::isA(type) || type == "HostsBased";
}

SynSym *HostsBasedSym::clone(const String &type) const {
	if (isA(type))
		return new HostsBasedSym(this->type(), theRec->clone());
	return RecSym::clone(type);
}

int HostsBasedSym::hostCount() const {
	if (ArraySym *a = getArraySym("hosts"))
		return a->count();
	else
		return 0;
}

NetAddrSym *HostsBasedSym::host(int idx) const {
	if (ArraySym *a = getArraySym("hosts")) {
		Assert(idx < a->count());
		return &(NetAddrSym&)(*a)[idx]->cast("addr");
	}
	return 0;
}

bool HostsBasedSym::hosts(Array<NetAddrSym*> &addrs) const {
	if (ArraySym *a = getArraySym("hosts")) {
		ArraySymExport(*a, "addr", addrs);
		return true;
	}
	return false;
}
