
/*
 * $Id: rng_test.cc,v 1.2 1999/09/24 18:05:52 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "polygraph.h"

#include <limits.h>
#include <stdlib.h>

#include "Rnd.h"
#include "RndPermut.h"
#include "gadgets.h"


static
int uncorr(int n) {
	//int exp;
	//const double x = INT_MAX * (2*frexp(::sin((double)n), &exp) - 1);
	const double x = INT_MAX * ::sin(n*(double)n);
	return (int)Min(fabs(x), (double)INT_MAX);
}

int main(int argc, char *argv[]) {
	int sampleCount = -1;
	if (argc != 2 || !isInt(argv[1], sampleCount)) {
		cerr << "usage: " << argv[0] << " <number_of_samples>" << endl;
		return 0;
	}

	RndGen rngCont;
	RndGen rngSeeded;
	RndGen rngPermut;

	for (int i = 1; i <= sampleCount; ++i) {
		rngSeeded.seed(i);
		rngPermut.seed(RndPermut(i));
		cout << random()
			<< ' ' << rngCont.ltrial()
			<< ' ' << rngSeeded.ltrial()
			<< ' ' << rngPermut.ltrial()
			<< ' ' << uncorr(i)
			<< endl;
	}

}
