
/*
 * $Id: ResourceUsage.cc,v 1.1.2.2 1999/11/08 06:37:16 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "xstd.h"

#include <errno.h>
#include <iostream.h>
#include <iomanip.h>

#include "ResourceUsage.h"


ResourceUsage ResourceUsage::Current() {
	ResourceUsage ru;
	ru.update();
	return ru;
}

ResourceUsage::ResourceUsage() {
	memset(this, 0, sizeof(*this));
}

Time ResourceUsage::sysTime() const {
	return Time(ru_stime);
}

Time ResourceUsage::userTime() const {
	return Time(ru_utime);
}

Time ResourceUsage::cpuTime() const {
	return userTime() + sysTime();
}

Size ResourceUsage::maxResSize() const {
	return ::MaxRss2Size(ru_maxrss);
}

long ResourceUsage::physPageFaults() const {
	return ru_majflt;
}

bool ResourceUsage::update() {
#if defined(HAVE_GETRUSAGE) && defined(RUSAGE_SELF)
	return ::getrusage(RUSAGE_SELF, this) == 0;
#else
	errno = EOPNOTSUPP;
	return false;
#endif
}

void ResourceUsage::report(ostream &os, const String &pfx) const {
	os <<
		pfx << "CPU Usage: " 
			<< sysTime() << " sys + " 
			<< userTime() << " user = " 
			<< cpuTime()
			<< endl;
	if (Size sz = maxResSize())
		os << pfx << "Maximum Resident Size: " << sz << endl;

	os 
		<< pfx << "Page faults with physical i/o: " << physPageFaults() << endl
		;
}
